@@ -146,27 +146,38 @@ class CHistoryManager {
 
 		if ($limit == 1) {
 			foreach ($items as $item) {
-				$values = DBfetchArray(DBselect(
-					'SELECT *'.
+				// Executing two subsequent queries individually for the sake of performance.
+
+				$clock_max = DBfetch(DBselect(
+					'SELECT MAX(h.clock)'.
 					' FROM '.self::getTableName($item['value_type']).' h'.
 					' WHERE h.itemid='.zbx_dbstr($item['itemid']).
-						' AND h.clock=('.
-							'SELECT MAX(h2.clock)'.
-							' FROM '.self::getTableName($item['value_type']).' h2'.
-							' WHERE h2.itemid='.zbx_dbstr($item['itemid']).
-								($period ? ' AND h2.clock>'.$period : '').
-						')'.
-					' ORDER BY h.ns DESC',
-					$limit
-				));
+						($period ? ' AND h.clock>'.$period : '')
+				), false);
 
-				if ($values) {
-					$results[$item['itemid']] = $values;
+				if ($clock_max) {
+					$clock_max = reset($clock_max);
+
+					if ($clock_max !== null) {
+						$values = DBfetchArray(DBselect(
+							'SELECT *'.
+							' FROM '.self::getTableName($item['value_type']).' h'.
+							' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+								' AND h.clock='.zbx_dbstr($clock_max).
+							' ORDER BY h.ns DESC',
+							$limit
+						));
+
+						if ($values) {
+							$results[$item['itemid']] = $values;
+						}
+					}
 				}
 			}
 		}
 		else {
 			foreach ($items as $item) {
+				// Cannot order by h.ns directly here due to performance issues.
 				$values = DBfetchArray(DBselect(
 					'SELECT *'.
 					' FROM '.self::getTableName($item['value_type']).' h'.
@@ -181,6 +192,11 @@ class CHistoryManager {
 					$clock = $values[$count - 1]['clock'];
 
 					if ($count == $limit + 1 && $values[$count - 2]['clock'] == $clock) {
+						/*
+						 * The last selected entries having the same clock means the selection (not just the order)
+						 * of the last entries is possibly wrong due to unordered by nanoseconds.
+						 */
+
 						do {
 							unset($values[--$count]);
 						} while ($values && $values[$count - 1]['clock'] == $clock);
@@ -220,8 +236,8 @@ class CHistoryManager {
 	}
 
 	/**
-	 * Returns the history value of the item at the given time. If no value exists at the given time, the function
-	 * will return the previous value.
+	 * Returns the history data of the item at the given time. If no data exists at the given time, the function will
+	 * return the previous data.
 	 *
 	 * The $item parameter must have the value_type and itemid properties set.
 	 *
@@ -231,7 +247,7 @@ class CHistoryManager {
 	 * @param int    $clock
 	 * @param int    $ns
 	 *
-	 * @return string|null  Value at specified time of first value before specified time. null if value is not found.
+	 * @return array|null  Item data at specified time of first data before specified time. null if data is not found.
 	 */
 	public function getValueAt(array $item, $clock, $ns) {
 		switch (self::getDataSourceType($item['value_type'])) {
@@ -304,7 +320,7 @@ class CHistoryManager {
 			$result = CElasticsearchHelper::query('POST', reset($endpoints), $query);
 
 			if (count($result) === 1 && is_array($result[0]) && array_key_exists('value', $result[0])) {
-				return $result[0]['value'];
+				return $result[0];
 			}
 		}
 
@@ -317,21 +333,21 @@ class CHistoryManager {
 	 * @see CHistoryManager::getValueAt
 	 */
 	private function getValueAtFromSql(array $item, $clock, $ns) {
-		$value = null;
+		$result = null;
 		$table = self::getTableName($item['value_type']);
 
-		$sql = 'SELECT value'.
+		$sql = 'SELECT *'.
 				' FROM '.$table.
 				' WHERE itemid='.zbx_dbstr($item['itemid']).
 					' AND clock='.zbx_dbstr($clock).
 					' AND ns='.zbx_dbstr($ns);
 
 		if (($row = DBfetch(DBselect($sql, 1))) !== false) {
-			$value = $row['value'];
+			$result = $row;
 		}
 
-		if ($value !== null) {
-			return $value;
+		if ($result !== null) {
+			return $result;
 		}
 
 		$max_clock = 0;
@@ -358,18 +374,18 @@ class CHistoryManager {
 		}
 
 		if ($max_clock == 0) {
-			return $value;
+			return $result;
 		}
 
 		if ($clock == $max_clock) {
-			$sql = 'SELECT value'.
+			$sql = 'SELECT *'.
 					' FROM '.$table.
 					' WHERE itemid='.zbx_dbstr($item['itemid']).
 						' AND clock='.zbx_dbstr($clock).
 						' AND ns<'.zbx_dbstr($ns);
 		}
 		else {
-			$sql = 'SELECT value'.
+			$sql = 'SELECT *'.
 					' FROM '.$table.
 					' WHERE itemid='.zbx_dbstr($item['itemid']).
 						' AND clock='.zbx_dbstr($max_clock).
@@ -377,10 +393,10 @@ class CHistoryManager {
 		}
 
 		if (($row = DBfetch(DBselect($sql, 1))) !== false) {
-			$value = $row['value'];
+			$result = $row;
 		}
 
-		return $value;
+		return $result;
 	}
 
 	/**
@@ -388,35 +404,27 @@ class CHistoryManager {
 	 *
 	 * The $item parameter must have the value_type, itemid and source properties set.
 	 *
-	 * @param array  $items        items to get aggregated values for
-	 * @param int    $time_from    minimal timestamp (seconds) to get data from
-	 * @param int    $time_to      maximum timestamp (seconds) to get data from
-	 * @param int    $width        graph width in pixels (is not required for pie charts)
+	 * @param array  $items      items to get aggregated values for
+	 * @param int    $time_from  minimal timestamp (seconds) to get data from
+	 * @param int    $time_to    maximum timestamp (seconds) to get data from
+	 * @param string $function   function for data aggregation
+	 * @param string $interval   aggregation interval in seconds
 	 *
 	 * @return array    history value aggregation for graphs
 	 */
-	public function getGraphAggregation(array $items, $time_from, $time_to, $width = null) {
-		if ($width !== null) {
-			$size = $time_to - $time_from;
-			$delta = $size - $time_from % $size;
-		}
-		else {
-			$size = null;
-			$delta = null;
-		}
-
+	public function getGraphAggregationByInterval(array $items, $time_from, $time_to, $function, $interval) {
 		$grouped_items = self::getItemsGroupedByStorage($items);
 
 		$results = [];
 		if (array_key_exists(ZBX_HISTORY_SOURCE_ELASTIC, $grouped_items)) {
-			$results += $this->getGraphAggregationFromElasticsearch($grouped_items[ZBX_HISTORY_SOURCE_ELASTIC],
-					$time_from, $time_to, $width, $size, $delta
+			$results = $this->getGraphAggregationByIntervalFromElasticsearch($grouped_items[ZBX_HISTORY_SOURCE_ELASTIC],
+				$time_from, $time_to, $function, $interval
 			);
 		}
 
 		if (array_key_exists(ZBX_HISTORY_SOURCE_SQL, $grouped_items)) {
-			$results += $this->getGraphAggregationFromSql($grouped_items[ZBX_HISTORY_SOURCE_SQL], $time_from, $time_to,
-					$width, $size, $delta
+			$results += $this->getGraphAggregationByIntervalFromSql($grouped_items[ZBX_HISTORY_SOURCE_SQL],
+				$time_from, $time_to, $function, $interval
 			);
 		}
 
@@ -424,11 +432,264 @@ class CHistoryManager {
 	}
 
 	/**
-	 * Elasticsearch specific implementation of getGraphAggregation.
+	 * Elasticsearch specific implementation of getGraphAggregationByInterval.
 	 *
-	 * @see CHistoryManager::getGraphAggregation
+	 * @see CHistoryManager::getGraphAggregationByInterval
 	 */
-	private function getGraphAggregationFromElasticsearch(array $items, $time_from, $time_to, $width, $size, $delta) {
+	private function getGraphAggregationByIntervalFromElasticsearch(array $items, $time_from, $time_to, $function,
+			$interval) {
+		$terms = [];
+
+		foreach ($items as $item) {
+			$terms[$item['value_type']][] = $item['itemid'];
+		}
+
+		$aggs = ['clock' => ['max' => ['field' => 'clock']]];
+
+		switch ($function) {
+			case GRAPH_AGGREGATE_MIN:
+				$aggs['value'] = ['min' => ['field' => 'value']];
+				break;
+			case GRAPH_AGGREGATE_MAX:
+				$aggs['value'] = ['max' => ['field' => 'value']];
+				break;
+			case GRAPH_AGGREGATE_AVG:
+				$aggs['value'] = ['avg' => ['field' => 'value']];
+				break;
+			case GRAPH_AGGREGATE_SUM:
+				$aggs['value'] = ['sum' => ['field' => 'value']];
+				break;
+			case GRAPH_AGGREGATE_FIRST:
+				$aggs['value'] = ['top_hits' => ['size' => 1, 'sort' => ['clock' => ['order' => 'asc']]]];
+				$aggs['clock'] = ['min' => ['field' => 'clock']];
+				break;
+			case GRAPH_AGGREGATE_LAST:
+				$aggs['value'] = ['top_hits' => ['size' => 1, 'sort' => ['clock' => ['order' => 'desc']]]];
+				break;
+		}
+
+		$query = [
+			'aggs' => [
+				'group_by_itemid' => [
+					'terms' => [
+						// Assure that aggregations for all terms are returned.
+						'size' => count($items),
+						'field' => 'itemid'
+					]
+				]
+			],
+			'size' => 0
+		];
+
+		// Clock value is divided by 1000 as it is stored as milliseconds.
+		$formula = '((doc[\'clock\'].date.getMillis()/1000) - ((doc[\'clock\'].date.getMillis()/1000)%params.interval))';
+
+		$query['aggs']['group_by_itemid']['aggs'] = [
+			'group_by_script' => [
+				'terms' => [
+					'size' => (($time_to - $time_from) / $interval) + 1,
+					'script' => [
+						'inline' => $formula,
+						'params' => [
+							'interval' => $interval
+						]
+					]
+				],
+				'aggs' => $aggs
+			]
+		];
+
+		$results = [];
+		foreach (self::getElasticsearchEndpoints(array_keys($terms)) as $type => $endpoint) {
+			$query['query']['bool']['must'] = [
+				[
+					'terms' => [
+						'itemid' => $terms[$type]
+					]
+				],
+				[
+					'range' => [
+						'clock' => [
+							'gte' => $time_from,
+							'lte' => $time_to
+						]
+					]
+				]
+			];
+
+			$data = CElasticsearchHelper::query('POST', $endpoint, $query);
+
+			foreach ($data['group_by_itemid']['buckets'] as $item) {
+				if (!is_array($item['group_by_script']) || !array_key_exists('buckets', $item['group_by_script'])
+					|| !is_array($item['group_by_script']['buckets'])) {
+					continue;
+				}
+
+				foreach ($item['group_by_script']['buckets'] as $point) {
+					$row = [
+						'itemid' => $item['key'],
+						'tick' => (int)$point['key'],
+						'count' => $point['doc_count'],
+						'clock' => (int)$point['clock']['value_as_string']
+					];
+
+					if ($function == GRAPH_AGGREGATE_FIRST || $function == GRAPH_AGGREGATE_LAST) {
+						$row['value'] = $point['value']['hits']['hits'][0]['_source']['value'];
+					}
+					else {
+						$row['value'] = array_key_exists('value', $point) ? $point['value']['value'] : null;
+					}
+
+					$results[$item['key']]['data'][] = $row;
+				}
+
+				if (array_key_exists($item['key'], $results)) {
+					$results[$item['key']]['source'] = 'history';
+				}
+			}
+		}
+
+		return $results;
+	}
+
+	/**
+	 * SQL specific implementation of getGraphAggregationByWidth.
+	 *
+	 * @see CHistoryManager::getGraphAggregationByInterval
+	 */
+	private function getGraphAggregationByIntervalFromSql(array $items, $time_from, $time_to, $function, $interval) {
+		$items_by_table = [];
+		foreach ($items as $item) {
+			$items_by_table[$item['value_type']][$item['source']][] = $item['itemid'];
+		}
+
+		$result = [];
+
+		foreach ($items_by_table as $value_type => $items_by_source) {
+			foreach ($items_by_source as $source => $itemids) {
+				$sql_select = ['itemid'];
+				$sql_group_by = ['itemid'];
+
+				$calc_field = zbx_dbcast_2bigint('clock').'-'.zbx_sql_mod(zbx_dbcast_2bigint('clock'), $interval);
+				$sql_select[] = $calc_field.' AS tick';
+				$sql_group_by[] = $calc_field;
+
+				if ($source === 'history') {
+					switch ($function) {
+						case GRAPH_AGGREGATE_MIN:
+							$sql_select[] = 'MIN(value) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_MAX:
+							$sql_select[] = 'MAX(value) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_AVG:
+							$sql_select[] = 'AVG(value) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_COUNT:
+							$sql_select[] = 'COUNT(*) AS count, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_SUM:
+							$sql_select[] = 'SUM(value) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_FIRST:
+							$sql_select[] = 'MIN(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_LAST:
+							$sql_select[] = 'MAX(clock) AS clock';
+							break;
+					}
+					$sql_from = ($value_type == ITEM_VALUE_TYPE_UINT64) ? 'history_uint' : 'history';
+				}
+				else {
+					switch ($function) {
+						case GRAPH_AGGREGATE_MIN:
+							$sql_select[] = 'MIN(value_min) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_MAX:
+							$sql_select[] = 'MAX(value_max) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_AVG:
+							$sql_select[] = 'AVG(value_avg) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_COUNT:
+							$sql_select[] = 'SUM(num) AS count, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_SUM:
+							$sql_select[] = '(value_avg * num) AS value, MAX(clock) AS clock';
+							$sql_group_by = array_merge($sql_group_by, ['value_avg', 'num']);
+							break;
+						case GRAPH_AGGREGATE_FIRST:
+							$sql_select[] = 'MIN(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_LAST:
+							$sql_select[] = 'MAX(clock) AS clock';
+							break;
+					}
+					$sql_from = ($value_type == ITEM_VALUE_TYPE_UINT64) ? 'trends_uint' : 'trends';
+				}
+
+				$sql = 'SELECT '.implode(', ', $sql_select).
+					' FROM '.$sql_from.
+					' WHERE '.dbConditionInt('itemid', $itemids).
+					' AND clock >= '.zbx_dbstr($time_from).
+					' AND clock <= '.zbx_dbstr($time_to).
+					' GROUP BY '.implode(', ', $sql_group_by);
+
+				if ($function == GRAPH_AGGREGATE_FIRST || $function == GRAPH_AGGREGATE_LAST) {
+					$sql = 'SELECT DISTINCT h.itemid, h.'.($source === 'history' ? 'value' : 'value_avg').' AS value, h.clock, hi.tick'.
+						' FROM '.$sql_from.' h'.
+						' JOIN('.$sql.') hi ON h.itemid = hi.itemid AND h.clock = hi.clock';
+				}
+
+				$sql_result = DBselect($sql);
+
+				while (($row = DBfetch($sql_result)) !== false) {
+					$result[$row['itemid']]['source'] = $source;
+					$result[$row['itemid']]['data'][] = $row;
+				}
+			}
+		}
+
+		return $result;
+	}
+
+	/**
+	 * Returns history value aggregation for graphs.
+	 *
+	 * The $item parameter must have the value_type, itemid and source properties set.
+	 *
+	 * @param array $items      items to get aggregated values for
+	 * @param int   $time_from  minimal timestamp (seconds) to get data from
+	 * @param int   $time_to    maximum timestamp (seconds) to get data from
+	 * @param int   $width      graph width in pixels (is not required for pie charts)
+	 *
+	 * @return array    history value aggregation for graphs
+	 */
+	public function getGraphAggregationByWidth(array $items, $time_from, $time_to, $width = null) {
+		$grouped_items = self::getItemsGroupedByStorage($items);
+
+		$results = [];
+		if (array_key_exists(ZBX_HISTORY_SOURCE_ELASTIC, $grouped_items)) {
+			$results += $this->getGraphAggregationByWidthFromElasticsearch($grouped_items[ZBX_HISTORY_SOURCE_ELASTIC],
+					$time_from, $time_to, $width
+			);
+		}
+
+		if (array_key_exists(ZBX_HISTORY_SOURCE_SQL, $grouped_items)) {
+			$results += $this->getGraphAggregationByWidthFromSql($grouped_items[ZBX_HISTORY_SOURCE_SQL],
+				$time_from, $time_to, $width
+			);
+		}
+
+		return $results;
+	}
+
+	/**
+	 * Elasticsearch specific implementation of getGraphAggregationByWidth.
+	 *
+	 * @see CHistoryManager::getGraphAggregationByWidth
+	 */
+	private function getGraphAggregationByWidthFromElasticsearch(array $items, $time_from, $time_to, $width) {
 		$terms = [];
 
 		foreach ($items as $item) {
@@ -490,7 +751,10 @@ class CHistoryManager {
 			'size' => 0
 		];
 
-		if ($width !== null && $size !== null && $delta !== null) {
+		if ($width !== null) {
+			$size = $time_to - $time_from;
+			$delta = $size - $time_from % $size;
+
 			// Additional grouping for line graphs.
 			$aggs['max_clock'] = [
 				'max' => [
@@ -544,7 +808,7 @@ class CHistoryManager {
 
 			$data = CElasticsearchHelper::query('POST', $endpoint, $query);
 
-			if ($width !== null && $size !== null && $delta !== null) {
+			if ($width !== null) {
 				foreach ($data['group_by_itemid']['buckets'] as $item) {
 					if (!is_array($item['group_by_script']) || !array_key_exists('buckets', $item['group_by_script'])
 							|| !is_array($item['group_by_script']['buckets'])) {
@@ -585,15 +849,18 @@ class CHistoryManager {
 	}
 
 	/**
-	 * SQL specific implementation of getGraphAggregation.
+	 * SQL specific implementation of getGraphAggregationByWidth.
 	 *
-	 * @see CHistoryManager::getGraphAggregation
+	 * @see CHistoryManager::getGraphAggregationByWidth
 	 */
-	private function getGraphAggregationFromSql(array $items, $time_from, $time_to, $width, $size, $delta) {
+	private function getGraphAggregationByWidthFromSql(array $items, $time_from, $time_to, $width) {
 		$group_by = 'itemid';
 		$sql_select_extra = '';
 
-		if ($width !== null && $size !== null && $delta !== null) {
+		if ($width !== null) {
+			$size = $time_to - $time_from;
+			$delta = $size - $time_from % $size;
+
 			// Required for 'group by' support of Oracle.
 			$calc_field = 'round('.$width.'*'.zbx_sql_mod(zbx_dbcast_2bigint('clock').'+'.$delta, $size)
 					.'/('.$size.'),0)';
